import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_swipe/flutter_swipe.dart';

import 'package:scrobbles/cubit/bottom_nav_cubit.dart';
import 'package:scrobbles/ui/screens/discoveries.dart';
import 'package:scrobbles/ui/screens/home.dart';
import 'package:scrobbles/ui/screens/settings.dart';
import 'package:scrobbles/ui/screens/statistics.dart';
import 'package:scrobbles/ui/widgets/app_bar.dart';
import 'package:scrobbles/ui/widgets/bottom_nav_bar.dart';

class SkeletonScreen extends StatefulWidget {
  const SkeletonScreen({super.key});

  @override
  State<SkeletonScreen> createState() => _SkeletonScreenState();
}

class _SkeletonScreenState extends State<SkeletonScreen> {
  @override
  Widget build(BuildContext context) {
    List<Widget> pageNavigation = <Widget>[
      ScreenHome(notifyParent: refresh),
      ScreenDiscoveries(notifyParent: refresh),
      ScreenStatistics(notifyParent: refresh),
      const ScreenSettings(),
    ];

    return Scaffold(
      appBar: StandardAppBar(notifyParent: refresh),
      extendBodyBehindAppBar: false,
      body: Swiper(
        itemCount: BlocProvider.of<BottomNavCubit>(context).pagesCount,
        itemBuilder: (BuildContext context, int index) {
          return pageNavigation.elementAt(index);
        },
        pagination: SwiperPagination(
          builder: SwiperCustomPagination(
            builder: (BuildContext context, SwiperPluginConfig config) {
              return BottomNavBar(swipeController: config.controller);
            },
          ),
        ),
        onIndexChanged: (newPageIndex) {
          BlocProvider.of<BottomNavCubit>(context).updateIndex(newPageIndex);
        },
        outer: true,
        loop: false,
      ),
      backgroundColor: Theme.of(context).colorScheme.background,
    );
  }

  refresh() {
    void rebuild(Element el) {
      el.markNeedsBuild();
      el.visitChildren(rebuild);
    }

    (context as Element).visitChildren(rebuild);
  }
}
